/*
 * A test for smblib ...
 */

#include <stdlib.h>
#include <string.h>
#include <rfcnb.h>
#include <smblib.h>
#include <tcp.h>

#include "util.h"

char data[65536] = "Now is the time for all birds to roost.";
char slosh[2]    = "\\";

int  verbose     = FALSE;
int  use_slosh   = FALSE;   /* Use a leading slash on addresses ... */

void usage (void)
{
  fprintf (stderr,
           "test_smblib [-dvl] [-b <slosh> -u <user> -s <file-service>] "
           "[-w <write-size] server [password]\n");
  exit (0);
}

/*
 * Copy a string with a slash on the front
 */
void smb_strncpy (char *dest, char *source, int destlen)
{
  if (use_slosh)
       strncpy (dest, slosh, destlen);
  else strncpy (dest, "", destlen);
  strncat (dest, source, destlen - strlen (dest));
}

char *SMB_Prots[] = {
     "PC NETWORK PROGRAM 1.0",
     "MICROSOFT NETWORKS 1.03",
     "MICROSOFT NETWORKS 3.0",
     "LANMAN1.0",
     "LM1.2X002",
     "LANMAN2.1",
     "NT LM 0.12",
     "NT LANMAN 1.0",
     NULL
   };

int main (int argc, char **argv)
{
  struct SMB_Tree *tree, *tree2;

  void *con, *file;
  int   opt, actual_size, i, written;
  int   lotc = FALSE, count;
  char  server[80], service[80], service_name[160];
  char  password[80], username[80];
  char  file_name[80];
  char  file_name2[80];

  struct SMB_CP_dirent dir_list[10];

  actual_size = 8192;
  strcpy (service, "msfax");
  strcpy (username, "fred");

  while ((opt = getopt (argc, argv, "b:s:w:u:ldv")) != EOF)
  {
    switch (opt)
    {
      case 's':
           strcpy (service, optarg);
           break;

      case 'w':
           actual_size = atoi (optarg) * 1024;
           break;

      case 'b':             /* set the slosh character */
           strncpy (slosh, optarg, sizeof (slosh) - 1);
           break;

      case 'u':             /* Pick up the user name */
           strncpy (username, optarg, sizeof (username) - 1);
           break;

      case 'l':             /* do a logon and tree connect ... */
           lotc = TRUE;
           break;

      case 'v':             /* Verbose? */
      case 'd':
           verbose = TRUE;
           dbug_init();
           break;

      default:
           usage();
    }
  }

  if (actual_size < 1024)
      actual_size = 1024;

  if (actual_size > sizeof(data))
      actual_size = sizeof(data);

  if (optind < argc)
  {
    strncpy (server, argv[optind], sizeof(server)-1);
    optind++;

    if (optind < argc)
         strncpy (password, argv[optind], sizeof(password)-1);
    else strcpy (password, "nurk");
  }
  else
    strcpy (server, "nemesis");

  SMB_Init();
  sock_init();

  sprintf (service_name, "%s%s%s%s%s", slosh, slosh, server, slosh, service);

  fprintf (stderr, "Connecting to %s ...\n", service_name);

  if (lotc)
  {
    /* We must do a connect server and a negotiate first
     */
    con = SMB_Connect_Server (NULL, server);
    if (!con)
    {
      SMB_ProcessError ("Unable to connect to server %s ...\n", server);
      return (1);
    }

    /* We need to negotiate a protocol better than PC NetWork Program
     */
    if (SMB_Negotiate (con, SMB_Prots) < 0)
    {
      SMB_ProcessError ("Unable to negotiate a protocol with server %s ...\n",
                        server);
      return (1);
    }

    /* Should check the protocol level ...
     */
    if (SMB_Get_Protocol (con) == SMB_P_NT1)
    {
      tree = SMB_Logon_And_TCon (con, NULL, username, password,
                                 service_name, "A:");
      if (!tree)
      {
        SMB_ProcessError ("Unable to logon and tree connect to server %s ...\n",
                          server);
        return (1);
      }
    }
    else       /* Try with a logon and then a TCON */
    {
      tree = SMB_Logon_And_TCon (con, NULL, username, password,
                                 service_name, "A:");
      if (!tree)
      {
        SMB_ProcessError ("Unable to logon to server %s ...\n", server);
        return (1);
      }
      fprintf (stderr, "Logged on and connected ...\n");
    }
  }
  else
  {
    con = SMB_Connect (NULL, &tree, service_name, "Guest", password);
    if (!con)
    {
      SMB_ProcessError ("Unable to connect:\n");
      return (1);
    }
  }

  fprintf (stderr,
           "Now we check to see if we can connect to the same file service.\n");

  tree2 = SMB_TreeConnect (con, NULL, service_name, password, "A:");
  if (!tree2)
       SMB_ProcessError ("Could not connect to tree: %s.\n", service_name);
  else fprintf (stderr, "Connected again to tree: %s.\n", service_name);

  strcpy (file_name, "netfax.q");

  fprintf (stderr, "Attempting to open \'%s\' ...\n", file_name);

  file = SMB_Open (tree, NULL, file_name, SMB_AMODE_OPENRW, SMB_FA_ORD);

  if (!file)   /* We failed, so try again, but print an error mess */
  {
    SMB_ProcessError ("Error opening file %s\n", file_name);

    fprintf (stderr, "Perhaps we need a \\ on the front. Trying again...\n");

    /* not put a slosh at the front and try again
     */
    use_slosh = TRUE;
    smb_strncpy (file_name2, file_name, sizeof (file_name2) - 1);
    use_slosh = FALSE;

    fprintf (stderr, "Attempting to open file \'%s\'.\n", file_name2);

    file = SMB_Open (tree, NULL, file_name2, SMB_AMODE_OPENRW, SMB_FA_ORD);
    if (file)
    {
      use_slosh = TRUE;
      fprintf (stderr, "--> Server needs %s in front of file names\n", slosh);
    }
  }

  /* Check how we did, maybe we need to try again with a slosh on the front
   */
  if (file)   /* Now write and close the file ... */
  {
    for (i = 0; i < actual_size; i++)
        data[i] = (char)(i & 0xFF);

    strncpy (data + actual_size - 20, "abcdefghijklmnopqrs", 19);

    fprintf (stderr, "Attempting to write %i bytes to the file ... \n",
             actual_size);

    written = SMB_Write (file, data, actual_size);

    fprintf (stderr, "Bytes written = %i\n", written);

    fprintf (stderr, "Now setting file pos to 512 and writing again ...\n");

    SMB_Lseek (file, 512, SEEK_SET);

    written = SMB_Write (file, data, actual_size);

    fprintf (stderr, "Bytes written = %i\n", written);

    fprintf (stderr, "Now read from start for 1024 bytes ...\n");

    SMB_Lseek (file, 0, SEEK_SET);

    for (i = 0; i < sizeof (data); i++)
        data[i] = 0xff;

    SMB_Read (file, data, 1024);

    fprintf (stderr, "Attempting to close the file just opened ...\n");
    SMB_Close (file);
  }
  else
    SMB_ProcessError ("Error opening file %s\n", file_name2);

  smb_strncpy (file_name, "xyzwertyaaa.dfgty", sizeof(file_name)-1);

  fprintf (stderr, "Attempting to create file %s ...\n", file_name);

  file = SMB_Create (tree, NULL, file_name, SMB_FA_ORD);

  if (file)   /* Now attempt to write the file ... */
  {
    fprintf (stderr, "The file was opened OK. So now write it ...\n");

    written = SMB_Write (file, data, actual_size);

    fprintf (stderr, "Now close the file just created ...\n");

    SMB_Close (file);
  }
  else
    SMB_ProcessError ("Error creating file %s\n", file_name);

  strcpy (file_name, "delete.txt");

  fprintf (stderr, "Attempting to delete the file '%s' ...\n", file_name);

  if (SMB_Delete (tree, file_name, SMB_FA_ORD) < 0)
  {
    SMB_ProcessError ("The file could not be deleted ... \n");

    fprintf (stderr, "Trying to delete %s%s ... \n", slosh, file_name);

    strcpy (file_name2, slosh);
    strncat (file_name2, file_name, sizeof(file_name2)-strlen(file_name));

    if (SMB_Delete (tree, file_name2, SMB_FA_ORD) < 0)
         SMB_ProcessError ("Still could not delete file %s ...\n",
                           file_name2);
    else fprintf (stderr, "%s Deleted now ...\n", file_name2);
  }
  else
    fprintf (stderr, "The file was deleted ... \n");

  smb_strncpy (file_name, "alongnamehurts.txt", sizeof(file_name)-1);

  fprintf (stderr, "Attempting to create '%s' ...\n", file_name);

  file = SMB_Create (tree, NULL, file_name, 0);

  if (!file)        /* It failed ... why? */
       fprintf (stderr, "Creating file '%s' failed ...\n", file_name);
  else fprintf (stderr, "  File created successfully.\n");

  smb_strncpy (file_name, "another-dir-name", sizeof(file_name)-1);

  fprintf (stderr, "Attempting to create directory \'%s\' ...\n", file_name);

  if (SMB_Create_Dir (tree, file_name) < 0)
       fprintf (stderr, "Creating directory \'%s\' failed ...\n", file_name);
  else fprintf (stderr, "  Directory created successfully.\n");

  fprintf (stderr, "Attempting to check the directory \'%s\' ...\n", file_name);

  if (SMB_Check_Dir (tree, file_name) < 0)
       fprintf (stderr, "Directory \'%s\' does not exist or is inaccessible ...\n",
                file_name);
  else fprintf (stderr, "  Directory checked successfully.\n");

  fprintf (stderr, "Attempting to delete directory \'%s\' ... \n", file_name);

  if (SMB_Delete_Dir (tree, file_name) < 0)
       fprintf (stderr, "Deleting directory \'%s\' failed ...\n", file_name);
  else fprintf (stderr, "  Directory deleted successfully.\n");

  /* Now do a directory listing ...
   */
  strcpy (file_name, "\\*.*");

  count = SMB_Search (tree, file_name, 0, dir_list, 10, "", 0);
  if (count < 0)
     SMB_ProcessError ("Directory listing failed ...\n");

  else           /* List the directory entries ... */
  {
    if (count == 0)
       fprintf (stderr, "\n  No files found.\n\n");
    else
    {
      fprintf (stderr, "  File Name            Size        Attribs  Date\n");
      fprintf (stderr, "  ---------            ----------  -------  ----\n");

      for (i = 0; i < count; i++)
      {
        fprintf (stderr, "  %-20s %-10lu  %-7s  %s\n",
                 dir_list[i].filename,
                 dir_list[i].size,
                 SMB_AttrToStr (dir_list[i].file_attributes, FALSE),
                 SMB_DOSTimToStr (dir_list[i].date_time));
      }
      fprintf (stderr, "\n");
    }
  }

  /* Should clean up here ... */

  fprintf (stderr, "Done\n");
  return (0);
}
